@@ -1,16 +1,41 @@
 import ast
+import inspect
 import logging
 import operator
 import re
 import time
+import warnings
 from functools import partial
-from typing import Any, Callable, ClassVar, Dict, List, Match, NamedTuple, Optional, Pattern, Sequence, Type
+from http.cookiejar import Cookie
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    ClassVar,
+    Dict,
+    List,
+    Match,
+    NamedTuple,
+    Optional,
+    Pattern,
+    Sequence,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 import requests.cookies
 
 from streamlink.cache import Cache
 from streamlink.exceptions import FatalPluginError, NoStreamsError, PluginError
-from streamlink.options import Arguments, Options
+from streamlink.options import Argument, Arguments, Options
+from streamlink.user_input import UserInputRequester
+
+
+if TYPE_CHECKING:  # pragma: no cover
+    from streamlink.session import Streamlink
+
 
 log = logging.getLogger(__name__)
 
@@ -50,6 +75,9 @@ NORMAL_PRIORITY = 20
 LOW_PRIORITY = 10
 NO_PRIORITY = 0
 
+_COOKIE_KEYS = \
+    "version", "name", "value", "port", "domain", "path", "secure", "expires", "discard", "comment", "comment_url", "rfc2109"
+
 
 def stream_weight(stream):
     for group, weights in QUALITY_WEIGTHS_EXTRA.items():
@@ -153,36 +181,53 @@ def parse_params(params: Optional[str] = None) -> Dict[str, Any]:
     return rval
 
 
-class UserInputRequester:
-    """
-    Base Class / Interface for requesting user input
-
-    eg. From the console
-    """
-    def ask(self, prompt):
-        """
-        Ask the user for a text input, the input is not sensitive
-        and can be echoed to the user
-
-        :param prompt: message to display when asking for the input
-        :return: the value the user input
-        """
-        raise NotImplementedError
-
-    def ask_password(self, prompt):
-        """
-        Ask the user for a text input, the input _is_ sensitive
-        and should be masked as the user gives the input
-
-        :param prompt: message to display when asking for the input
-        :return: the value the user input
-        """
-        raise NotImplementedError
-
-
 class Matcher(NamedTuple):
     pattern: Pattern
     priority: int
+    name: Optional[str] = None
+
+
+MType = TypeVar("MType")
+
+
+class _MCollection(List[MType]):
+    def __init__(self, **kwargs) -> None:
+        super().__init__(**kwargs)
+        self._names: Dict[str, MType] = {}
+
+    def __getitem__(self, item):
+        return self._names[item] if type(item) is str else super().__getitem__(item)
+
+
+class Matchers(_MCollection[Matcher]):
+    def register(self, matcher: Matcher) -> None:
+        super().insert(0, matcher)
+        if matcher.name:
+            if matcher.name in self._names:
+                raise ValueError(f"A matcher named '{matcher.name}' has already been registered")
+            self._names[matcher.name] = matcher
+
+
+class Matches(_MCollection[Optional[Match]]):
+    def update(self, matchers: Matchers, value: str) -> Tuple[Optional[Pattern], Optional[Match]]:
+        matches = [(matcher, matcher.pattern.match(value)) for matcher in matchers]
+
+        self.clear()
+        self.extend(match for matcher, match in matches)
+        self._names.clear()
+        self._names.update((matcher.name, match) for matcher, match in matches if matcher.name)
+
+        return next(((matcher.pattern, match) for matcher, match in matches if match is not None), (None, None))
+
+
+# Add front- and back-wrappers to the deprecated plugin's method resolution order (see Plugin.__new__)
+class PluginWrapperMeta(type):
+    def mro(cls):
+        # cls.__base__ is the PluginWrapperFront which is based on the deprecated plugin class
+        mro = list(cls.__base__.__mro__)
+        # cls is the PluginWrapperBack and needs to be inserted after the deprecated plugin class
+        mro.insert(2, cls)
+        return mro
 
 
 class Plugin:
@@ -190,19 +235,31 @@ class Plugin:
     Plugin base class for retrieving streams and metadata from the URL specified.
     """
 
-    matchers: ClassVar[Optional[List[Matcher]]] = None
+    matchers: ClassVar[Optional[Matchers]] = None
     """
-    The list of plugin matchers (URL pattern + priority).
-    Use the :meth:`streamlink.plugin.pluginmatcher` decorator for initializing this list.
+    The list of plugin matchers (URL pattern + priority + optional name).
+    This list supports matcher lookups both by matcher index, as well as matcher name, if defined.
+
+    Use the :func:`pluginmatcher` decorator to initialize plugin matchers.
     """
 
-    matches: Sequence[Optional[Match]]
-    """A tuple of :class:`re.Match` results of all defined matchers"""
+    arguments: ClassVar[Optional[Arguments]] = None
+    """
+    The plugin's :class:`Arguments <streamlink.options.Arguments>` collection.
 
-    matcher: Optional[Pattern]
+    Use the :func:`pluginargument` decorator to initialize plugin arguments.
+    """
+
+    matches: Matches
+    """
+    A list of optional :class:`re.Match` results of all defined matchers.
+    This list supports match lookups both by the respective matcher index, as well as matcher name, if defined.
+    """
+
+    matcher: Optional[Pattern] = None
     """A reference to the compiled :class:`re.Pattern` of the first matching matcher"""
 
-    match: Optional[Match]
+    match: Optional[Match] = None
     """A reference to the :class:`re.Match` result of the first matching matcher"""
 
     # plugin metadata attributes
@@ -215,35 +272,74 @@ class Plugin:
     category: Optional[str] = None
     """Metadata 'category' attribute: name of a game being played, a music genre, etc."""
 
-    cache = None
-    logger = None
-    module = "unknown"
     options = Options()
-    arguments = Arguments()
-    session = None
-    _url: Optional[str] = None
-    _user_input_requester = None
+    _url: str = ""
 
     # deprecated
     can_handle_url: Callable[[str], bool]
     # deprecated
     priority: Callable[[str], int]
 
-    @classmethod
-    def bind(cls, session, module, user_input_requester=None):
-        cls.cache = Cache(filename="plugin-cache.json",
-                          key_prefix=module)
-        cls.logger = logging.getLogger("streamlink.plugins." + module)
-        cls.module = module
-        cls.session = session
-        if user_input_requester is not None:
-            if isinstance(user_input_requester, UserInputRequester):
-                cls._user_input_requester = user_input_requester
-            else:
-                raise RuntimeError("user-input-requester must be an instance of UserInputRequester")
+    # Handle deprecated plugin constructors which only take the url argument
+    def __new__(cls, *args, **kwargs):
+        # Ignore plugins without custom constructors or wrappers
+        if cls.__init__ is Plugin.__init__ or hasattr(cls, "_IS_DEPRECATED_PLUGIN_WRAPPER"):
+            return super().__new__(cls)
+
+        # Ignore custom constructors which have a formal "session" parameter or a variable positional parameter
+        sig = inspect.signature(cls.__init__).parameters
+        if "session" in sig or any(param.kind == inspect.Parameter.VAR_POSITIONAL for param in sig.values()):
+            return super().__new__(cls)
+
+        # Wrapper class which overrides the very first constructor in the MRO
+        # noinspection PyAbstractClass
+        class PluginWrapperFront(cls):
+            _IS_DEPRECATED_PLUGIN_WRAPPER = True
+
+            # The __module__ value needs to be copied
+            __module__ = cls.__module__
+
+            def __init__(self, session, url):
+                # Take any arguments, but only pass the URL to the custom constructor of the deprecated plugin
+                # noinspection PyArgumentList
+                super().__init__(url)
+                warnings.warn(
+                    f"Initialized {self.module} plugin with deprecated constructor",
+                    FutureWarning,
+                )
+
+        # Wrapper class which comes after the deprecated plugin in the MRO
+        # noinspection PyAbstractClass
+        class PluginWrapperBack(PluginWrapperFront, metaclass=PluginWrapperMeta):
+            def __init__(self, *_, **__):
+                # Take any arguments from the super() call of the constructor of the deprecated plugin,
+                # but pass the right args and keywords to the Plugin constructor
+                super().__init__(*args, **kwargs)
+
+        return cls.__new__(PluginWrapperBack, *args, **kwargs)
+
+    def __init__(self, session: "Streamlink", url: str):
+        """
+        :param session: The Streamlink session instance
+        :param url: The input URL used for finding and resolving streams
+        """
+
+        modulename = self.__class__.__module__
+        self.module = modulename.split(".")[-1]
+        self.logger = logging.getLogger(modulename)
+        self.cache = Cache(
+            filename="plugin-cache.json",
+            key_prefix=self.module,
+        )
+
+        self.session: "Streamlink" = session
+        self.matches = Matches()
+        self.url: str = url
+
+        self.load_cookies()
 
     @property
-    def url(self) -> Optional[str]:
+    def url(self) -> str:
         """
         The plugin's input URL.
         Setting a new value will automatically update the :attr:`matches`, :attr:`matcher` and :attr:`match` data.
@@ -255,21 +351,8 @@ class Plugin:
     def url(self, value: str):
         self._url = value
 
-        matches = [(pattern, pattern.match(value)) for pattern, priority in self.matchers or []]
-        self.matches = tuple(m for p, m in matches)
-        self.matcher, self.match = next(((p, m) for p, m in matches if m is not None), (None, None))
-
-    def __init__(self, url: str):
-        """
-        :param str url: URL that the plugin will operate on
-        """
-
-        self.url = url
-
-        try:
-            self.load_cookies()
-        except RuntimeError:
-            pass  # unbound cannot load
+        if self.matchers:
+            self.matcher, self.match = self.matches.update(self.matchers, value)
 
     @classmethod
     def set_option(cls, key, value):
@@ -281,7 +364,7 @@ class Plugin:
 
     @classmethod
     def get_argument(cls, key):
-        return cls.arguments.get(key)
+        return cls.arguments and cls.arguments.get(key)
 
     @classmethod
     def stream_weight(cls, stream):
@@ -299,24 +382,6 @@ class Plugin:
 
         return stream_types
 
-    @classmethod
-    def broken(cls, issue=None):
-        def func(*args, **kwargs):
-            msg = (
-                "This plugin has been marked as broken. This is likely due to "
-                "changes to the service preventing a working implementation. "
-            )
-
-            if issue:
-                msg += "More info: https://github.com/streamlink/streamlink/issues/{0}".format(issue)
-
-            raise PluginError(msg)
-
-        def decorator(*args, **kwargs):
-            return func
-
-        return decorator
-
     def streams(self, stream_types=None, sorting_excludes=None):
         """
         Attempts to extract available streams.
@@ -463,7 +528,7 @@ class Plugin:
             id=self.get_id(),
             author=self.get_author(),
             category=self.get_category(),
-            title=self.get_title()
+            title=self.get_title(),
         )
 
     def get_id(self) -> Optional[str]:
@@ -480,8 +545,8 @@ class Plugin:
 
     def save_cookies(
         self,
-        cookie_filter: Optional[Callable] = None,
-        default_expires: int = 60 * 60 * 24 * 7
+        cookie_filter: Optional[Callable[[Cookie], bool]] = None,
+        default_expires: int = 60 * 60 * 24 * 7,
     ) -> List[str]:
         """
         Store the cookies from :attr:`session.http` in the plugin cache until they expire. The cookies can be filtered
@@ -493,31 +558,30 @@ class Plugin:
         :return: list of the saved cookie names
         """
 
-        if not self.session or not self.cache:
-            raise RuntimeError("Cannot cache cookies in unbound plugin")
-
         cookie_filter = cookie_filter or (lambda c: True)
         saved = []
 
         for cookie in filter(cookie_filter, self.session.http.cookies):
             cookie_dict = {}
-            for attr in ("version", "name", "value", "port", "domain", "path", "secure", "expires", "discard",
-                         "comment", "comment_url", "rfc2109"):
-                cookie_dict[attr] = getattr(cookie, attr, None)
+            for key in _COOKIE_KEYS:
+                cookie_dict[key] = getattr(cookie, key, None)
             cookie_dict["rest"] = getattr(cookie, "rest", getattr(cookie, "_rest", None))
 
             expires = default_expires
-            if cookie_dict['expires']:
-                expires = int(cookie_dict['expires'] - time.time())
-            key = "__cookie:{0}:{1}:{2}:{3}".format(cookie.name,
-                                                    cookie.domain,
-                                                    cookie.port_specified and cookie.port or "80",
-                                                    cookie.path_specified and cookie.path or "*")
+            if cookie_dict["expires"]:
+                expires = int(cookie_dict["expires"] - time.time())
+            key = "__cookie:{0}:{1}:{2}:{3}".format(
+                cookie.name,
+                cookie.domain,
+                cookie.port_specified and cookie.port or "80",
+                cookie.path_specified and cookie.path or "*",
+            )
             self.cache.set(key, cookie_dict, expires)
             saved.append(cookie.name)
 
-        if saved:
-            self.logger.debug("Saved cookies: {0}".format(", ".join(saved)))
+        if saved:  # pragma: no branch
+            self.logger.debug(f"Saved cookies: {', '.join(saved)}")
+
         return saved
 
     def load_cookies(self) -> List[str]:
@@ -527,9 +591,6 @@ class Plugin:
         :return: list of the restored cookie names
         """
 
-        if not self.session or not self.cache:
-            raise RuntimeError("Cannot load cached cookies in unbound plugin")
-
         restored = []
 
         for key, value in self.cache.get_all().items():
@@ -538,8 +599,9 @@ class Plugin:
                 self.session.http.cookies.set_cookie(cookie)
                 restored.append(cookie.name)
 
-        if restored:
-            self.logger.debug("Restored cookies: {0}".format(", ".join(restored)))
+        if restored:  # pragma: no branch
+            self.logger.debug(f"Restored cookies: {', '.join(restored)}")
+
         return restored
 
     def clear_cookies(self, cookie_filter: Optional[Callable] = None) -> List[str]:
@@ -552,9 +614,6 @@ class Plugin:
         :return: list of the removed cookie names
         """
 
-        if not self.session or not self.cache:
-            raise RuntimeError("Cannot clear cached cookies in unbound plugin")
-
         cookie_filter = cookie_filter or (lambda c: True)
         removed = []
 
@@ -568,36 +627,135 @@ class Plugin:
 
         return removed
 
-    def input_ask(self, prompt):
-        if self._user_input_requester:
+    def input_ask(self, prompt: str) -> str:
+        user_input_requester: Optional[UserInputRequester] = self.session.get_option("user-input-requester")
+        if user_input_requester:
             try:
-                return self._user_input_requester.ask(prompt)
-            except OSError as e:
-                raise FatalPluginError("User input error: {0}".format(e))
-            except NotImplementedError:  # ignore this and raise a FatalPluginError
-                pass
+                return user_input_requester.ask(prompt)
+            except OSError as err:
+                raise FatalPluginError(f"User input error: {err}")
         raise FatalPluginError("This plugin requires user input, however it is not supported on this platform")
 
-    def input_ask_password(self, prompt):
-        if self._user_input_requester:
+    def input_ask_password(self, prompt: str) -> str:
+        user_input_requester: Optional[UserInputRequester] = self.session.get_option("user-input-requester")
+        if user_input_requester:
             try:
-                return self._user_input_requester.ask_password(prompt)
-            except OSError as e:
-                raise FatalPluginError("User input error: {0}".format(e))
-            except NotImplementedError:  # ignore this and raise a FatalPluginError
-                pass
+                return user_input_requester.ask_password(prompt)
+            except OSError as err:
+                raise FatalPluginError(f"User input error: {err}")
         raise FatalPluginError("This plugin requires user input, however it is not supported on this platform")
 
 
-def pluginmatcher(pattern: Pattern, priority: int = NORMAL_PRIORITY) -> Callable[[Type[Plugin]], Type[Plugin]]:
-    matcher = Matcher(pattern, priority)
+def pluginmatcher(
+    pattern: Pattern,
+    priority: int = NORMAL_PRIORITY,
+    name: Optional[str] = None,
+) -> Callable[[Type[Plugin]], Type[Plugin]]:
+    """
+    Decorator for plugin URL matchers.
+
+    A matcher consists of a compiled regular expression pattern for the plugin's input URL,
+    a priority value and an optional name.
+    The priority value determines which plugin gets chosen by
+    :meth:`Streamlink.resolve_url <streamlink.Streamlink.resolve_url>` if multiple plugins match the input URL.
+    The matcher name can be used for accessing it and its matching result when multiple matchers are defined.
+
+    Plugins must at least have one matcher. If multiple matchers are defined, then the first matching one
+    according to the order of which they have been defined (top to bottom) will be responsible for setting the
+    :attr:`Plugin.matcher` and :attr:`Plugin.match` attributes on the :class:`Plugin` instance.
+    The :attr:`Plugin.matchers` and :attr:`Plugin.matches` attributes are affected by all defined matchers,
+    and both support referencing matchers and matches by matcher index and name.
+
+    .. code-block:: python
+
+        import re
+
+        from streamlink.plugin import HIGH_PRIORITY, Plugin, pluginmatcher
+
+
+        @pluginmatcher(re.compile("https?://example:1234/(?:foo|bar)/(?P<name>[^/]+)"))
+        @pluginmatcher(priority=HIGH_PRIORITY, pattern=re.compile(\"\"\"
+            https?://(?:
+                 sitenumberone
+                |adifferentsite
+                |somethingelse
+            )
+            /.+\\.m3u8
+        \"\"\", re.VERBOSE))
+        class MyPlugin(Plugin):
+            ...
+    """
+
+    matcher = Matcher(pattern, priority, name)
+
+    def decorator(cls: Type[Plugin]) -> Type[Plugin]:
+        if not issubclass(cls, Plugin):
+            raise TypeError(f"{cls.__name__} is not a Plugin")
+        if cls.matchers is None:
+            cls.matchers = Matchers()
+        cls.matchers.register(matcher)
+
+        return cls
+
+    return decorator
+
+
+def pluginargument(
+    name: str,
+    required: bool = False,
+    requires: Optional[Union[str, Sequence[str]]] = None,
+    prompt: Optional[str] = None,
+    sensitive: bool = False,
+    argument_name: Optional[str] = None,
+    dest: Optional[str] = None,
+    is_global: bool = False,
+    **options,
+) -> Callable[[Type[Plugin]], Type[Plugin]]:
+    """
+    Decorator for plugin arguments. Takes the same arguments as :class:`streamlink.options.Argument`.
+
+    .. code-block:: python
+
+        from streamlink.plugin import Plugin, pluginargument
+
+
+        @pluginargument(
+            "username",
+            requires=["password"],
+            metavar="EMAIL",
+            help="The username for your account.",
+        )
+        @pluginargument(
+            "password",
+            sensitive=True,
+            metavar="PASSWORD",
+            help="The password for your account.",
+        )
+        class MyPlugin(Plugin):
+            ...
+
+    This will add the ``--myplugin-username`` and ``--myplugin-password`` arguments to the CLI,
+    assuming the plugin's module name is ``myplugin``.
+    """
+
+    arg = Argument(
+        name,
+        required=required,
+        requires=requires,
+        prompt=prompt,
+        sensitive=sensitive,
+        argument_name=argument_name,
+        dest=dest,
+        is_global=is_global,
+        **options,
+    )
 
     def decorator(cls: Type[Plugin]) -> Type[Plugin]:
         if not issubclass(cls, Plugin):
             raise TypeError(f"{repr(cls)} is not a Plugin")
-        if cls.matchers is None:
-            cls.matchers = []
-        cls.matchers.insert(0, matcher)
+        if cls.arguments is None:
+            cls.arguments = Arguments()
+        cls.arguments.add(arg)
 
         return cls
 
@@ -608,4 +766,5 @@ __all__ = [
     "HIGH_PRIORITY", "NORMAL_PRIORITY", "LOW_PRIORITY", "NO_PRIORITY",
     "Plugin",
     "Matcher", "pluginmatcher",
+    "pluginargument",
 ]
