@@ -1,5 +1,5 @@
 """
-$description Japanese live streaming and video hosting social platform.
+$description Japanese live-streaming and video hosting social platform.
 $url openrec.tv
 $type live, vod
 """
@@ -7,16 +7,29 @@ $type live, vod
 import logging
 import re
 
-from streamlink.plugin import Plugin, PluginArgument, PluginArguments, pluginmatcher
+from streamlink.plugin import Plugin, pluginargument, pluginmatcher
 from streamlink.plugin.api import validate
 from streamlink.stream.hls import HLSStream
 
+
 log = logging.getLogger(__name__)
 
 
 @pluginmatcher(re.compile(
-    r"https?://(?:www\.)?openrec\.tv/(?:live|movie)/(?P<id>[^/]+)"
+    r"https?://(?:www\.)?openrec\.tv/(?:live|movie)/(?P<id>[^/]+)",
 ))
+@pluginargument(
+    "email",
+    requires=["password"],
+    metavar="EMAIL",
+    help="The email associated with your openrectv account, required to access any openrectv stream.",
+)
+@pluginargument(
+    "password",
+    sensitive=True,
+    metavar="PASSWORD",
+    help="An openrectv account password to use with --openrectv-email.",
+)
 class OPENRECtv(Plugin):
     _stores_re = re.compile(r"window.stores\s*=\s*({.*?});", re.DOTALL | re.MULTILINE)
     _config_re = re.compile(r"window.sharedConfig\s*=\s*({.*?});", re.DOTALL | re.MULTILINE)
@@ -26,11 +39,11 @@ class OPENRECtv(Plugin):
     login_url = "https://www.openrec.tv/viewapp/v4/mobile/user/login"
 
     _info_schema = validate.Schema({
-        validate.optional("id"): validate.text,
-        validate.optional("title"): validate.text,
-        validate.optional("movie_type"): validate.text,
+        validate.optional("id"): str,
+        validate.optional("title"): str,
+        validate.optional("movie_type"): str,
         validate.optional("onair_status"): validate.any(None, int),
-        validate.optional("public_type"): validate.text,
+        validate.optional("public_type"): str,
         validate.optional("media"): {
             "url": validate.any(None, validate.url()),
             "url_public": validate.any(None, validate.url()),
@@ -39,7 +52,7 @@ class OPENRECtv(Plugin):
         validate.optional("subs_trial_media"): {
             "url": validate.any(None, validate.url()),
             "url_ull": validate.any(None, validate.url()),
-        }
+        },
     })
 
     _subscription_schema = validate.Schema({
@@ -47,38 +60,20 @@ class OPENRECtv(Plugin):
         validate.optional("data"): {
             "items": [{
                 "media": {
-                    "url": validate.any(None, validate.url())
-                }
-            }]
-        }
+                    "url": validate.any(None, validate.url()),
+                },
+            }],
+        },
     })
 
     _login_schema = validate.Schema({
-        validate.optional("error_message"): validate.text,
+        validate.optional("error_message"): str,
         "status": int,
-        validate.optional("data"): object
+        validate.optional("data"): object,
     })
 
-    arguments = PluginArguments(
-        PluginArgument(
-            "email",
-            requires=["password"],
-            metavar="EMAIL",
-            help="""
-            The email associated with your openrectv account,
-            required to access any openrectv stream.
-            """),
-        PluginArgument(
-            "password",
-            sensitive=True,
-            metavar="PASSWORD",
-            help="""
-            An openrectv account password to use with --openrectv-email.
-            """)
-    )
-
-    def __init__(self, url):
-        super().__init__(url)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.video_id = None
 
     def login(self, email, password):
@@ -94,7 +89,7 @@ class OPENRECtv(Plugin):
         url = self.movie_info_url.format(id=self.video_id)
         res = self.session.http.get(url, headers={
             "access-token": self.session.http.cookies.get("access_token"),
-            "uuid": self.session.http.cookies.get("uuid")
+            "uuid": self.session.http.cookies.get("uuid"),
         })
         data = self.session.http.json(res, schema=self._info_schema)
 
@@ -108,7 +103,7 @@ class OPENRECtv(Plugin):
         url = self.subscription_info_url.format(id=self.video_id)
         res = self.session.http.get(url, headers={
             "access-token": self.session.http.cookies.get("access_token"),
-            "uuid": self.session.http.cookies.get("uuid")
+            "uuid": self.session.http.cookies.get("uuid"),
         })
         data = self.session.http.json(res, schema=self._subscription_schema)
 
@@ -129,7 +124,7 @@ class OPENRECtv(Plugin):
             return mdata["title"]
 
     def _get_streams(self):
-        self.video_id = self.url.rsplit('/', 1)[-1]
+        self.video_id = self.url.rsplit("/", 1)[-1]
         if self.get_option("email") and self.get_option("password"):
             self.login(self.get_option("email"), self.get_option("password"))
         mdata = self._get_movie_data()
@@ -156,7 +151,7 @@ class OPENRECtv(Plugin):
             if m3u8_file is not None:
                 yield from HLSStream.parse_variant_playlist(
                     self.session,
-                    m3u8_file
+                    m3u8_file,
                 ).items()
 
         else:
