@@ -11,7 +11,7 @@ import re
 from datetime import datetime
 from urllib.parse import urlparse
 
-from isodate import LOCAL as LOCALTIMEZONE
+from isodate import LOCAL as LOCALTIMEZONE  # type: ignore[import]
 
 from streamlink.plugin import Plugin, PluginError, pluginmatcher
 from streamlink.plugin.api import useragents, validate
@@ -19,6 +19,7 @@ from streamlink.stream.dash import DASHStream
 from streamlink.stream.hls import HLSStream
 from streamlink.utils.url import update_qsd
 
+
 log = logging.getLogger(__name__)
 
 
@@ -34,12 +35,9 @@ class Pluzz(Plugin):
     GEO_URL = "https://geoftv-a.akamaihd.net/ws/edgescape.json"
     API_URL = "https://player.webservices.francetelevisions.fr/v1/videos/{video_id}"
 
-    _re_ftv_player_videos = re.compile(r"window\.FTVPlayerVideos\s*=\s*(?P<json>\[{.+?}])\s*;\s*(?:$|var)", re.DOTALL)
-    _re_player_load = re.compile(r"""player\.load\s*\(\s*{\s*src\s*:\s*(['"])(?P<video_id>.+?)\1\s*}\s*\)\s*;""")
-
     def _get_streams(self):
         self.session.http.headers.update({
-            "User-Agent": useragents.CHROME
+            "User-Agent": useragents.CHROME,
         })
         CHROME_VERSION = re.compile(r"Chrome/(\d+)").search(useragents.CHROME).group(1)
 
@@ -47,9 +45,9 @@ class Pluzz(Plugin):
         country_code = self.session.http.get(self.GEO_URL, schema=validate.Schema(
             validate.parse_json(),
             {"reponse": {"geo_info": {
-                "country_code": str
+                "country_code": str,
             }}},
-            validate.get(("reponse", "geo_info", "country_code"))
+            validate.get(("reponse", "geo_info", "country_code")),
         ))
         log.debug(f"Country: {country_code}")
 
@@ -62,27 +60,32 @@ class Pluzz(Plugin):
                     validate.all(
                         validate.xml_xpath_string(".//script[contains(text(),'window.FTVPlayerVideos')][1]/text()"),
                         str,
-                        validate.transform(self._re_ftv_player_videos.search),
+                        validate.regex(re.compile(
+                            r"window\.FTVPlayerVideos\s*=\s*(?P<json>\[{.+?}])\s*;\s*(?:$|var)",
+                            re.DOTALL,
+                        )),
                         validate.get("json"),
                         validate.parse_json(),
                         [{"videoId": str}],
-                        validate.get((0, "videoId"))
+                        validate.get((0, "videoId")),
                     ),
                     validate.all(
                         validate.xml_xpath_string(".//script[contains(text(),'new Magnetoscope')][1]/text()"),
                         str,
-                        validate.transform(self._re_player_load.search),
-                        validate.get("video_id")
+                        validate.regex(re.compile(
+                            r"""player\.load\s*\(\s*{\s*src\s*:\s*(?P<q>['"])(?P<video_id>.+?)(?P=q)\s*}\s*\)\s*;""",
+                        )),
+                        validate.get("video_id"),
                     ),
                     validate.all(
                         validate.xml_xpath_string(".//*[@id][contains(@class,'francetv-player-wrapper')][1]/@id"),
-                        str
+                        str,
                     ),
                     validate.all(
                         validate.xml_xpath_string(".//*[@data-id][@class='magneto'][1]/@data-id"),
-                        str
-                    )
-                )
+                        str,
+                    ),
+                ),
             ))
         except PluginError:
             pass
@@ -100,7 +103,7 @@ class Pluzz(Plugin):
             "browser": "chrome",
             "browser_version": CHROME_VERSION,
             "os": "ios",
-            "gmt": datetime.now(tz=LOCALTIMEZONE).strftime("%z")
+            "gmt": datetime.now(tz=LOCALTIMEZONE).strftime("%z"),
         })
         video_format, token_url, url, self.title = self.session.http.get(api_url, schema=validate.Schema(
             validate.parse_json(),
@@ -109,27 +112,27 @@ class Pluzz(Plugin):
                     "workflow": validate.any("token-akamai", "dai"),
                     "format": validate.any("dash", "hls"),
                     "token": validate.url(),
-                    "url": validate.url()
+                    "url": validate.url(),
                 },
                 "meta": {
-                    "title": str
-                }
+                    "title": str,
+                },
             },
             validate.union_get(
                 ("video", "format"),
                 ("video", "token"),
                 ("video", "url"),
-                ("meta", "title")
-            )
+                ("meta", "title"),
+            ),
         ))
 
         data_url = update_qsd(token_url, {
-            "url": url
+            "url": url,
         })
         video_url = self.session.http.get(data_url, schema=validate.Schema(
             validate.parse_json(),
             {"url": validate.url()},
-            validate.get("url")
+            validate.get("url"),
         ))
 
         if video_format == "dash":
