@@ -1,10 +1,20 @@
+"""
+$description Live TV channels and video on-demand service from TV4, a Swedish free-to-air broadcaster.
+$url tv4play.se
+$url fotbollskanalen.se
+$type live, vod
+$region Sweden
+$notes Only non-premium streams are supported
+"""
+
 import logging
 import re
 from urllib.parse import urljoin
 
 from streamlink.plugin import Plugin, PluginError, pluginmatcher
 from streamlink.plugin.api import validate
-from streamlink.stream import HLSStream
+from streamlink.stream.hls import HLSStream
+
 
 log = logging.getLogger(__name__)
 
@@ -27,10 +37,10 @@ class TV4Play(Plugin):
     _meta_schema = validate.Schema(
         {
             "metadata": {
-                "title": validate.text
+                "title": str,
             },
-            "mediaUri": validate.text
-        }
+            "mediaUri": str,
+        },
     )
 
     @property
@@ -49,7 +59,7 @@ class TV4Play(Plugin):
         try:
             res = self.session.http.get(
                 self.api_assets.format(self.get_video_id),
-                params=params
+                params=params,
             )
         except Exception as e:
             if "404 Client Error" in str(e):
