@@ -10,23 +10,20 @@ import re
 from html import unescape as html_unescape
 from urllib.parse import urlparse
 
-from streamlink.plugin import Plugin, pluginargument, pluginmatcher
+from streamlink.plugin import Plugin, pluginmatcher
 from streamlink.plugin.api import validate
 from streamlink.stream.dash import DASHStream
 from streamlink.stream.ffmpegmux import MuxedStream
 from streamlink.stream.hls import HLSStream
 from streamlink.stream.http import HTTPStream
 
+
 log = logging.getLogger(__name__)
 
 
 @pluginmatcher(re.compile(
-    r"https?://(player\.vimeo\.com/video/\d+|(www\.)?vimeo\.com/.+)"
+    r"https?://(player\.vimeo\.com/video/\d+|(www\.)?vimeo\.com/.+)",
 ))
-@pluginargument(
-    "mux-subtitles",
-    is_global=True,
-)
 class Vimeo(Plugin):
     _config_url_re = re.compile(r'(?:"config_url"|\bdata-config-url)\s*[:=]\s*(".+?")')
     _config_re = re.compile(r"var\s+config\s*=\s*({.+?})\s*;")
@@ -47,16 +44,16 @@ class Vimeo(Plugin):
         {
             "request": {
                 "files": {
-                    validate.optional("dash"): {"cdns": {validate.text: {"url": validate.url()}}},
-                    validate.optional("hls"): {"cdns": {validate.text: {"url": validate.url()}}},
+                    validate.optional("dash"): {"cdns": {str: {"url": validate.url()}}},
+                    validate.optional("hls"): {"cdns": {str: {"url": validate.url()}}},
                     validate.optional("progressive"): validate.all(
-                        [{"url": validate.url(), "quality": validate.text}]
+                        [{"url": validate.url(), "quality": str}],
                     ),
                 },
                 validate.optional("text_tracks"): validate.all(
-                    [{"url": validate.text, "lang": validate.text}]
+                    [{"url": str, "lang": str}],
                 ),
-            }
+            },
         },
     )
     _player_schema = validate.Schema(
@@ -104,7 +101,7 @@ class Vimeo(Plugin):
             for stream in videos.get("progressive", [])
         )
 
-        if self.get_option("mux_subtitles") and data["request"].get("text_tracks"):
+        if self.session.get_option("mux-subtitles") and data["request"].get("text_tracks"):
             substreams = {
                 s["lang"]: HTTPStream(self.session, "https://vimeo.com" + s["url"])
                 for s in data["request"]["text_tracks"]
